<!-- As a heading -->
<nav class="navbar navbar-expand-lg bg-body-tertiary">
  <div class="container-fluid">
    <a class="navbar-brand" href="#"><img src="/images/gns-logo.png" alt="GNS Registrar" style="height:3em;"></a>
    <button class="navbar-toggler" type="button" data-bs-toggle="collapse" data-bs-target="#navbarSupportedContent" aria-controls="navbarSupportedContent" aria-expanded="false" aria-label="Toggle navigation">
      <span class="navbar-toggler-icon"></span>
    </button>
    <div class="collapse navbar-collapse" id="navbarSupportedContent">
      <ul class="navbar-nav me-auto mb-2 mb-lg-0">
        <li class="nav-item">
          <a class="nav-link" aria-current="page" href="/">Home</a>
        </li>
        <!--<li class="nav-item">
          <a class="nav-link" href="/about">About</a>
        </li>-->
      </ul>
    </div>
    <div>GNUnet GNS Registrar</div>
  </div>
</nav>
<!--<div class="alert alert-warning" role="alert">
  <p class="mb-0">
  Test
  </p>
</div>-->
