<footer class="fs-6 text-sm-end mt-4 me-2">
  <span><a href="https://git.gnunet.org/gnunet-gns-registrar.git">Browse Git Repository</a></span><br/>
  <!--<span><a href="/terms">Terms</a></span><br/>
  <span><a href="/privacy">Privacy</a></span><br/>
  <span><a href="/imprint">Imprint</a></span><br/>-->
  <span>{{.version}}</span><br/>
</footer>
